@@ -708,6 +708,7 @@ bool CowFile::setUnused(_i64 unused_start, _i64 unused_end)
 		Server->Log("FSCTL_SET_ZERO_DATA failed (setting unused image range) with errno " + convert(static_cast<int>(GetLastError())), LL_WARNING);
 		return false;
 	}
+	setBitmapRange(unused_start, unused_end, false);
 	return true;
 #else
 	if(!trim_warned)
@@ -827,7 +828,8 @@ bool CowFile::trimUnused(_i64 fs_offset, _i64 trim_blocksize, ITrimCallback* tri
 				unused_end = filesize;
 			}
 			
-			if(hasBitmapRangeNarrow(unused_start, unused_end, trim_blocksize_bytes))
+			if(hasBitmapRangeNarrow(unused_start, unused_end, trim_blocksize_bytes)
+				&& unused_end>unused_start)
 			{
 				if(!setUnused(unused_start, unused_end))
 				{
@@ -848,7 +850,8 @@ bool CowFile::trimUnused(_i64 fs_offset, _i64 trim_blocksize, ITrimCallback* tri
 		int64 unused_start = fs_offset + unused_start_block*bitmap_blocksize;
 		int64 unused_end = filesize;
 		
-		if(hasBitmapRangeNarrow(unused_start, unused_end, trim_blocksize_bytes))
+		if(hasBitmapRangeNarrow(unused_start, unused_end, trim_blocksize_bytes)
+			&& unused_end>unused_start)
 		{
 			if(!setUnused(unused_start, unused_end))
 			{
