@@ -39,8 +39,7 @@
 #include "win_network_cost.h"
 #else
 #include "lin_ver.h"
-std::string getSysVolumeCached(std::string &mpath){ return ""; }
-std::string getEspVolumeCached(std::string &mpath){ return ""; }
+#include "lin_sysvol.h"
 #endif
 #include "../client_version.h"
 
@@ -78,7 +77,7 @@ void ClientConnector::CMD_ADD_IDENTITY(const std::string &identity, const std::s
 			ServerIdentityMgr::loadServerIdentities();
 			if( ServerIdentityMgr::checkServerIdentity(identity) )
 			{
-				if(ServerIdentityMgr::hasPublicKey(identity))
+				if(ServerIdentityMgr::hasPublicKey(identity, true))
 				{
 					tcpstack.Send(pipe, "needs certificate");
 				}
@@ -131,6 +130,8 @@ void ClientConnector::CMD_GET_CHALLENGE(const std::string &identity, const std::
 	}
 
 	std::string clientsubname;
+	bool with_enc = false;
+	std::string ecies_pubkey;
 	if (cmd.size() > 14)
 	{
 		std::string s_params = cmd.substr(14);
@@ -138,13 +139,55 @@ void ClientConnector::CMD_GET_CHALLENGE(const std::string &identity, const std::
 		ParseParamStrHttp(s_params, &params);
 
 		clientsubname = params["clientsubname"];
+		ecies_pubkey = base64_decode_dash(params["ecies_pubkey"]);
+		with_enc = params["with_enc"] == "1";
+	}
+
+	bool local_encrypted = false;
+	bool local_compressed = false;
+	IECDHKeyExchange* shared_key_exchange = NULL;
+	std::string ret_params;
+
+	if (!internet_conn)
+	{
+		std::auto_ptr<ISettingsReader> settings(
+			Server->createFileSettingsReader("urbackup/data/settings.cfg"));
+
+		local_encrypted = settings->getValue("local_encrypt", true);
+		local_compressed = settings->getValue("local_compress", true);
+	}
+
+	if (with_enc)
+	{
+		if (local_compressed)
+		{
+#ifndef NO_ZSTD_COMPRESSION
+			ret_params += "&compress=zstd";
+#else
+			ret_params += "&compress=zlib";
+#endif
+		}
+
+		if (local_encrypted)
+		{
+			shared_key_exchange = crypto_fak->createECDHKeyExchange();
+			ret_params += "&pubkey_ecdh233k1=" + base64_encode_dash(shared_key_exchange->getPublicKey());
+		}
+
+		if (!ret_params.empty())
+			ret_params[0] = '?';
+	}
+	else if (local_encrypted)
+	{
+		Server->Log("Client requires encryption which server does not offer", LL_ERROR);
+		return;
 	}
 
 	IScopedLock lock(ident_mutex);
 	std::string challenge = Server->secureRandomString(30)+"-"+convert(Server->getTimeSeconds())+"-"+convert(Server->getTimeMS());
-	challenges[std::make_pair(identity, clientsubname)]=challenge;
+	challenges[std::make_pair(identity, clientsubname)]=SChallenge(challenge, shared_key_exchange, local_compressed);
 
-	tcpstack.Send(pipe, challenge);
+	tcpstack.Send(pipe, challenge + ret_params);
 }
 
 void ClientConnector::CMD_SIGNATURE(const std::string &identity, const std::string &cmd)
@@ -177,16 +220,16 @@ void ClientConnector::CMD_SIGNATURE(const std::string &identity, const std::stri
 
 	IScopedLock lock(ident_mutex);
 
-	std::map<std::pair<std::string, std::string>, std::string>::iterator challenge_it = challenges.find(std::make_pair(identity, clientsubname));
+	std::map<std::pair<std::string, std::string>, SChallenge>::iterator challenge_it = challenges.find(std::make_pair(identity, clientsubname));
 
-	if(challenge_it==challenges.end() || challenge_it->second.empty())
+	if(challenge_it==challenges.end() || challenge_it->second.challenge_str.empty())
 	{
 		Server->Log("Signature error: No challenge", LL_ERROR);
 		tcpstack.Send(pipe, "no challenge");
 		return;
 	}
 
-	const std::string& challenge = challenge_it->second;
+	const std::string& challenge = challenge_it->second.challenge_str;
 	
 
 	std::string pubkey = base64_decode_dash(params["pubkey"]);
@@ -194,10 +237,20 @@ void ClientConnector::CMD_SIGNATURE(const std::string &identity, const std::stri
 	std::string signature = base64_decode_dash(params["signature"]);
 	std::string signature_ecdsa409k1 = base64_decode_dash(params["signature_ecdsa409k1"]);
 	std::string session_identity = params["session_identity"];
+	std::string pubkey_ecdh233k1 = base64_decode_dash(params["pubkey_ecdh233k1"]);
+	std::string signature_ecdh233k1 = base64_decode_dash(params["signature_ecdh233k1"]);
+	std::string secret_session_key = base64_decode_dash(params["secret_session_key"]);
+	std::string signature_ecies = base64_decode_dash(params["signature_ecies"]);
+	std::string ecdh_shared_key;
+	std::string secret_session_key_decrypted;
 
-	if(!ServerIdentityMgr::hasPublicKey(identity))
+	if(!ServerIdentityMgr::hasPublicKey(identity, false))
 	{
-		ServerIdentityMgr::setPublicKeys(identity, SPublicKeys(pubkey, pubkey_ecdsa409k1));
+		if (!ServerIdentityMgr::setPublicKeys(identity, SPublicKeys(pubkey, pubkey_ecdsa409k1)))
+		{
+			tcpstack.Send(pipe, "pubkey fingerprint mismatch");
+			return;
+		}
 	}
 
 	SPublicKeys pubkeys = ServerIdentityMgr::getPublicKeys(identity);
@@ -205,10 +258,31 @@ void ClientConnector::CMD_SIGNATURE(const std::string &identity, const std::stri
 	if( (!pubkeys.ecdsa409k1_key.empty() && crypto_fak->verifyData(pubkeys.ecdsa409k1_key, challenge, signature_ecdsa409k1))
 		|| (pubkeys.ecdsa409k1_key.empty() && !pubkeys.dsa_key.empty() && crypto_fak->verifyDataDSA(pubkeys.dsa_key, challenge, signature)) )
 	{
-		ServerIdentityMgr::addSessionIdentity(session_identity, endpoint_name);
-		ServerIdentityMgr::setPublicKeys(identity, SPublicKeys(pubkey, pubkey_ecdsa409k1));
-		tcpstack.Send(pipe, "ok");
-		challenges.erase(challenge_it);
+		if (challenge_it->second.shared_key_exchange==NULL ||
+			(crypto_fak->verifyData(pubkeys.ecdsa409k1_key, challenge + pubkey_ecdh233k1, signature_ecdh233k1) &&
+				!(ecdh_shared_key = challenge_it->second.shared_key_exchange->getSharedKey(pubkey_ecdh233k1)).empty() &&
+				!(secret_session_key_decrypted = crypto_fak->decryptAuthenticatedAES(secret_session_key,
+					ecdh_shared_key, 1)).empty()
+			) )
+		{
+			if (!ServerIdentityMgr::setPublicKeys(identity, SPublicKeys(pubkey, pubkey_ecdsa409k1)))
+			{
+				tcpstack.Send(pipe, "pubkey fingerprint mismatch (2)");
+				return;
+			}
+
+			ServerIdentityMgr::addSessionIdentity(session_identity, endpoint_name,
+				secret_session_key_decrypted);
+			
+			tcpstack.Send(pipe, "ok");
+			Server->destroy(challenge_it->second.shared_key_exchange);
+			challenges.erase(challenge_it);
+		}
+		else
+		{
+			Server->Log("Encryption session key exchange failed", LL_ERROR);
+			tcpstack.Send(pipe, "encryption session key exchange failed");
+		}		
 	}
 	else
 	{
@@ -607,6 +681,7 @@ void ClientConnector::CMD_WAIT_FOR_INDEX(const std::string &cmd)
 		Server->Log("Wait for async index " + async_id, LL_DEBUG);
 		state = CCSTATE_START_FILEBACKUP_ASYNC;
 		++it->second.refcount;
+		it->second.last_update = Server->getTimeMS();
 		curr_result_id = it->second.result_id;
 		IndexThread::refResult(curr_result_id);
 	}
@@ -747,7 +822,7 @@ void ClientConnector::CMD_SET_INCRINTERVAL(const std::string &cmd)
 void ClientConnector::CMD_GET_BACKUPDIRS(const std::string &cmd)
 {
 	IDatabase *db=Server->getDatabase(Server->getThreadID(), URBACKUPDB_CLIENT);
-	IQuery *q=db->Prepare("SELECT id,name,path,tgroup,optional FROM backupdirs WHERE symlinked=0");
+	IQuery *q=db->Prepare("SELECT id, name, path, tgroup, optional, server_default FROM backupdirs WHERE symlinked=0");
 	int timeoutms=300;
 	db_results res=q->Read(&timeoutms);
 
@@ -769,6 +844,7 @@ void ClientConnector::CMD_GET_BACKUPDIRS(const std::string &cmd)
 			cdir.set("path", res[i]["path"]);
 			int tgroup = watoi(res[i]["tgroup"]);
 			cdir.set("group", tgroup%c_group_size);
+			cdir.set("server_default", watoi(res[i]["server_default"]));
 
 			if (tgroup < 0) continue;
 			
@@ -787,16 +863,7 @@ void ClientConnector::CMD_GET_BACKUPDIRS(const std::string &cmd)
 
 			int flags = watoi(res[i]["optional"]);
 			
-			std::vector<std::pair<int, std::string> > flag_mapping;
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_Optional, "optional"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_FollowSymlinks, "follow_symlinks"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_SymlinksOptional, "symlinks_optional"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_OneFilesystem, "one_filesystem"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_RequireSnapshot, "require_snapshot"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_KeepFiles, "keep"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_ShareHashes, "share_hashes"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_Required, "required"));
-			flag_mapping.push_back(std::make_pair(EBackupDirFlag_IncludeDirectorySymlinks, "include_dir_symlinks"));
+			std::vector<std::pair<int, std::string> > flag_mapping = getFlagStrMapping();
 			
 
 			std::string str_flags;
@@ -831,11 +898,12 @@ void ClientConnector::CMD_SAVE_BACKUPDIRS(const std::string &cmd, str_map &param
 {
 	if(last_capa & DONT_ALLOW_CONFIG_PATHS)
 	{
+		Server->Log("Changing paths to backup is disabled", LL_WARNING);
 		tcpstack.Send(pipe, "FAILED");
 		return;
 	}
 
-	if(saveBackupDirs(params))
+	if(saveBackupDirs(params, false, 0))
 	{
 		tcpstack.Send(pipe, "OK");
 	}
@@ -931,6 +999,8 @@ void ClientConnector::CMD_BACKUP_FAILED(const std::string & cmd)
 		status_updated = true;
 	}
 
+	IndexThread::execute_postbackup_hook("postfilebackup_failed", atoi(params["group"].c_str()), params["clientsubname"]);
+
 	exit_backup_immediate(1);
 }
 
@@ -1236,15 +1306,20 @@ void ClientConnector::CMD_CHANNEL(const std::string &cmd, IScopedLock *g_lock, c
 		}
 #endif
 
-		g_lock->relock(backup_mutex);
-
 		std::string token;
 
-		std::string s_params=cmd.substr(9);
+		std::string s_params = cmd.substr(9);
 		str_map params;
 		ParseParamStrHttp(s_params, &params);
-		int capa=watoi(params["capa"]);
-		token=params["token"];
+		int capa = watoi(params["capa"]);
+		token = params["token"];
+
+		if (params["startup"] == "1")
+		{
+			tcpstack.Send(pipe, "STARTUP timestamp=" + convert(startup_timestamp));
+		}
+
+		g_lock->relock(backup_mutex);
 
 		channel_pipes.push_back(SChannel(pipe, internet_conn, endpoint_name, token,
 			&make_fileserv, identity, capa, watoi(params["restore_version"]), params["virtual_client"]));
@@ -1460,6 +1535,12 @@ void ClientConnector::CMD_FULL_IMAGE(const std::string &cmd, bool ident_ok)
 				return;
 			}
 		}
+#ifndef _WIN32
+		else
+		{
+			image_inf.image_letter = mapLinuxDev(image_inf.image_letter);
+		}
+#endif
 
 		int running_jobs = 2;
 		if (params.find("running_jobs") != params.end())
@@ -1561,6 +1642,10 @@ void ClientConnector::CMD_INCR_IMAGE(const std::string &cmd, bool ident_ok)
 			image_inf.no_shadowcopy=false;
 			image_inf.clientsubname = params["clientsubname"];
 
+#ifndef _WIN32
+			image_inf.image_letter = mapLinuxDev(image_inf.image_letter);
+#endif
+
 			str_map::iterator f_cbitmapsize = params.find("cbitmapsize");
 			if (f_cbitmapsize != params.end())
 			{
@@ -1620,20 +1705,18 @@ void ClientConnector::CMD_INCR_IMAGE(const std::string &cmd, bool ident_ok)
 
 			state = CCSTATE_IMAGE_HASHDATA;
 
-			size_t bufpos = 0;
 			IFile* datafile = hashdatafile;
 			_u32* dataleft = &hashdataleft;
-			while(bufpos<tcpstack.getBuffersize())
+			while(tcpstack.getBuffersize()>0)
 			{
-				_u32 towrite = (std::min)(static_cast<_u32>(tcpstack.getBuffersize() - bufpos), *dataleft);
-				if(datafile->Write(tcpstack.getBuffer()+ bufpos, towrite)!= towrite)
+				_u32 towrite = (std::min)(static_cast<_u32>(tcpstack.getBuffersize()), *dataleft);
+				if(datafile->Write(tcpstack.getBuffer(), towrite)!= towrite)
 				{
 					Server->Log("Error writing to data temporary file in CMD_INCR_IMAGE", LL_ERROR);
 					do_quit=true;
 					return;
 				}
 				
-				bufpos += towrite;
 				*dataleft -= towrite;
 
 				tcpstack.removeFront(towrite);
@@ -1685,6 +1768,16 @@ void ClientConnector::CMD_MBR(const std::string &cmd)
 		std::string mpath;
 		dl=getEspVolumeCached(mpath);
 	}
+#ifndef _WIN32
+	else if (dl == "C" || dl == "C:")
+	{
+		dl = getRootVol();
+	}
+	else if(params.find("disk_path")!=params.end())
+	{
+		dl= mapLinuxDev(params["disk_path"]);
+	}
+#endif
 
 	bool b=false;
 	std::string errmsg;
@@ -2561,7 +2654,7 @@ void ClientConnector::CMD_CAPA(const std::string &cmd)
 	tcpstack.Send(pipe, "FILE=2&FILE2=1&IMAGE=1&UPDATE=1&MBR=1&FILESRV=3&SET_SETTINGS=1&IMAGE_VER=1&CLIENTUPDATE=2&ASYNC_INDEX=1"
 		"&CLIENT_VERSION_STR="+EscapeParamString((client_version_str))+"&OS_VERSION_STR="+EscapeParamString(os_version_str)+
 		"&ALL_VOLUMES="+EscapeParamString(win_volumes)+"&ETA=1&CDP=0&ALL_NONUSB_VOLUMES="+EscapeParamString(win_nonusb_volumes)+"&EFI=1"
-		"&FILE_META=1&SELECT_SHA=1&PHASH=1&RESTORE="+restore+"&CLIENT_BITMAP=1&CMD=2&SYMBIT=1&WTOKENS=1&OS_SIMPLE=windows"
+		"&FILE_META=1&SELECT_SHA=1&PHASH=1&RESTORE="+restore+"&RESTORE_VER=1&CLIENT_BITMAP=1&CMD=2&SYMBIT=1&WTOKENS=1&FILESRVTUNNEL=1&OS_SIMPLE=windows"
 		"&clientuid="+EscapeParamString(clientuid)+conn_metered+ send_prev_cbitmap + imm_backup);
 #else
 
@@ -2573,12 +2666,18 @@ void ClientConnector::CMD_CAPA(const std::string &cmd)
 	std::string os_simple = "unknown";
 #endif
 
+	std::string image_args = "&IMAGE=0";
+	if(!trim(IndexThread::get_snapshot_script_location("create_volume_snapshot", clientsubname)).empty())
+	{
+		image_args = "&IMAGE=1&REQ_PREV_CBITMAP=1";
+	}
+
 
 	std::string os_version_str=get_lin_os_version();
-	tcpstack.Send(pipe, "FILE=2&FILE2=1&FILESRV=3&SET_SETTINGS=1&CLIENTUPDATE=2&ASYNC_INDEX=1"
+	tcpstack.Send(pipe, "FILE=2&FILE2=1&FILESRV=3&SET_SETTINGS=1&IMAGE_VER=1&CLIENTUPDATE=2&ASYNC_INDEX=1"
 		"&CLIENT_VERSION_STR="+EscapeParamString((client_version_str))+"&OS_VERSION_STR="+EscapeParamString(os_version_str)
-		+"&ETA=1&CPD=0&FILE_META=1&SELECT_SHA=1&PHASH=1&RESTORE="+restore+"&CMD=2&SYMBIT=1&WTOKENS=1&OS_SIMPLE="+os_simple
-		+"&clientuid=" + EscapeParamString(clientuid) + imm_backup);
+		+"&ETA=1&CPD=0&EFI=1&FILE_META=1&SELECT_SHA=1&PHASH=1&RESTORE="+restore+"&RESTORE_VER=1&CLIENT_BITMAP=1&CMD=2&SYMBIT=1&WTOKENS=1&FILESRVTUNNEL=1&OS_SIMPLE="+os_simple
+		+"&clientuid=" + EscapeParamString(clientuid) + imm_backup + image_args);
 #endif
 }
 
@@ -2764,7 +2863,14 @@ void ClientConnector::CMD_FILE_RESTORE(const std::string& cmd)
 		return;
 	}
 
+	bool client_token_encrypted = true;
 	std::string client_token = (params["client_token"]);
+	if (client_token.empty()
+		&& params.find("client_token_d") != params.end())
+	{
+		client_token = params["client_token_d"];
+		client_token_encrypted = false;
+	}
 	std::string server_token=params["server_token"];
 	int64 restore_id=watoi64(params["id"]);
 	int64 status_id=watoi64(params["status_id"]);
@@ -2788,7 +2894,8 @@ void ClientConnector::CMD_FILE_RESTORE(const std::string& cmd)
 		restore_process_id = ++curr_backup_running_id;
 	}
 
-	if (crypto_fak != NULL)
+	if (crypto_fak != NULL
+		&& client_token_encrypted)
 	{
 		std::auto_ptr<ISettingsReader> access_keys(
 			Server->createFileSettingsReader("urbackup/access_keys.properties"));
@@ -2821,7 +2928,7 @@ void ClientConnector::CMD_FILE_RESTORE(const std::string& cmd)
 			Server->Log("Error opening urbackup/access_keys.properties", LL_ERROR);
 		}
 	}
-	else
+	else if(client_token_encrypted)
 	{
 		Server->Log("Error decrypting server access token. Crypto_fak not loaded.", LL_ERROR);
 	}
@@ -2966,4 +3073,10 @@ void ClientConnector::CMD_WRITE_TOKENS(const std::string& cmd)
 
 	tcpstack.Send(pipe, "ASYNC-async_id=" + bytesToHex(async_id));
 }
-	
\ No newline at end of file
+
+void ClientConnector::CMD_GET_CLIENTNAME(const std::string& cmd)
+{
+	std::string name = IndexThread::getFileSrv()->getServerName();
+	tcpstack.Send(pipe, "name="+EscapeParamString(name));
+}
+	
