@@ -39,8 +39,8 @@ ClientDAO::~ClientDAO()
 void ClientDAO::prepareQueries()
 {
 	q_get_files=db->Prepare("SELECT data, num, generation FROM files WHERE name=? AND tgroup=?", false);
-	q_add_files=db->Prepare("INSERT OR REPLACE INTO files (name, tgroup, num, data) VALUES (?,?,?,?)", false);
-	q_get_dirs=db->Prepare("SELECT name, path, id, optional, tgroup, symlinked, server_default, reset_keep FROM backupdirs", false);
+	q_add_files=db->Prepare("INSERT OR REPLACE INTO files (name, tgroup, num, data, generation) VALUES (?,?,?,?,?)", false);
+	q_get_dirs=db->Prepare("SELECT name, path, id, optional, tgroup, symlinked, server_default, reset_keep FROM backupdirs ORDER BY id ASC", false);
 	q_remove_all=db->Prepare("DELETE FROM files", false);
 	q_get_changed_dirs=db->Prepare("SELECT id, name FROM mdirs WHERE name GLOB ? UNION SELECT id, name FROM mdirs_backup WHERE name GLOB ?", false);
 	q_modify_files=db->Prepare("UPDATE files SET data=?, num=?, generation=? WHERE name=? AND tgroup=? AND generation=?", false);
@@ -322,7 +322,7 @@ GUID randomGuid()
 	return ret;
 }
 
-void ClientDAO::addFiles(std::string path, int tgroup, const std::vector<SFileAndHash> &data)
+void ClientDAO::addFiles(std::string path, int tgroup, const std::vector<SFileAndHash> &data, int64 target_generation)
 {
 	size_t ds;
 	char *buffer=constructData(data, ds);
@@ -330,6 +330,7 @@ void ClientDAO::addFiles(std::string path, int tgroup, const std::vector<SFileAn
 	q_add_files->Bind(tgroup);
 	q_add_files->Bind(ds);
 	q_add_files->Bind(buffer, (_u32)ds);
+	q_add_files->Bind(target_generation);
 	q_add_files->Write();
 	q_add_files->Reset();
 	delete []buffer;
@@ -469,6 +470,8 @@ std::vector<SShadowCopy> ClientDAO::getShadowcopies(void)
 
 int ClientDAO::addShadowcopy(const SShadowCopy &sc)
 {
+	DBScopedSynchronous sync_db(db);
+
 	q_insert_shadowcopy->Bind((char*)&sc.vssid, sizeof(GUID) );
 	q_insert_shadowcopy->Bind((char*)&sc.ssetid, sizeof(GUID) );
 	q_insert_shadowcopy->Bind(sc.target);
@@ -487,6 +490,8 @@ int ClientDAO::addShadowcopy(const SShadowCopy &sc)
 
 int ClientDAO::modShadowcopyRefCount(int id, int m)
 {
+	DBScopedSynchronous sync_db(db);
+
 	q_get_shadowcopy_refcount->Bind(id);
 	db_results res=q_get_shadowcopy_refcount->Read();
 	q_get_shadowcopy_refcount->Reset();
@@ -505,6 +510,8 @@ int ClientDAO::modShadowcopyRefCount(int id, int m)
 
 void ClientDAO::deleteShadowcopy(int id)
 {
+	DBScopedSynchronous sync_db(db);
+
 	q_remove_shadowcopies->Bind(id);
 	q_remove_shadowcopies->Write();
 	q_remove_shadowcopies->Reset();
@@ -970,3 +977,18 @@ void ClientDAO::resetAllHardlinks(void)
 	}
 	q_resetAllHardlinks->Write();
 }
+
+std::vector<std::pair<int, std::string> > getFlagStrMapping()
+{
+	std::vector<std::pair<int, std::string> > flag_mapping;
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_Optional, "optional"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_FollowSymlinks, "follow_symlinks"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_SymlinksOptional, "symlinks_optional"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_OneFilesystem, "one_filesystem"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_RequireSnapshot, "require_snapshot"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_KeepFiles, "keep"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_ShareHashes, "share_hashes"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_Required, "required"));
+	flag_mapping.push_back(std::make_pair(EBackupDirFlag_IncludeDirectorySymlinks, "include_dir_symlinks"));
+	return flag_mapping;
+}
